@@ -33,6 +33,14 @@
  */
 /* copyright --> */
 #include "DownloadEngine.h"
+
+#include <signal.h>
+
+#include <cstring>
+#include <cerrno>
+#include <algorithm>
+#include <numeric>
+
 #ifdef ENABLE_ASYNC_DNS
 #include "AsyncNameResolver.h"
 #endif // ENABLE_ASYNC_DNS
@@ -54,11 +62,18 @@
 #include "ServerStatMan.h"
 #include "CookieStorage.h"
 #include "A2STR.h"
-#include <signal.h>
-#include <cstring>
-#include <algorithm>
-#include <numeric>
-#include <cerrno>
+#include "DNSCache.h"
+#include "AuthConfigFactory.h"
+#include "AuthConfig.h"
+#include "Request.h"
+
+#include "BtRegistry.h"
+#include "BtContext.h"
+#include "PeerStorage.h"
+#include "PieceStorage.h"
+#include "BtAnnounce.h"
+#include "BtRuntime.h"
+#include "BtProgressInfoFile.h"
 
 namespace aria2 {
 
@@ -399,7 +414,10 @@ void AsyncNameResolverEntry::process(fd_set* rfdsPtr, fd_set* wfdsPtr)
 DownloadEngine::DownloadEngine():logger(LogFactory::getInstance()),
 				 _haltRequested(false),
 				 _noWait(false),
-				 _cookieStorage(new CookieStorage())
+				 _refreshInterval(DEFAULT_REFRESH_INTERVAL),
+				 _cookieStorage(new CookieStorage()),
+				 _btRegistry(new BtRegistry()),
+				 _dnsCache(new SimpleDNSCache())
 {
 #ifdef HAVE_EPOLL
 
@@ -473,7 +491,8 @@ void DownloadEngine::run() {
   Time cp;
   cp.setTimeInSec(0);
   while(!commands.empty() || !_routineCommands.empty()) {
-    if(cp.elapsed(1)) {
+    if(cp.elapsed(_refreshInterval)) {
+      _refreshInterval = DEFAULT_REFRESH_INTERVAL;
       cp.reset();
       executeCommand(commands, Command::STATUS_ALL);
     } else {
@@ -801,10 +820,14 @@ void DownloadEngine::afterEachIteration()
     logger->notice(_("Shutdown sequence commencing... Press Ctrl-C again for emergency shutdown."));
     requestHalt();
     globalHaltRequested = 2;
+    setNoWait(true);
+    setRefreshInterval(0);
   } else if(globalHaltRequested == 3) {
     logger->notice(_("Emergency shutdown sequence commencing..."));
     _requestGroupMan->forceHalt();
     globalHaltRequested = 4;
+    setNoWait(true);
+    setRefreshInterval(0);
   }
 }
 
@@ -939,6 +962,38 @@ void DownloadEngine::poolSocket
   poolSocket(ipaddr, port, e);
 }
 
+void DownloadEngine::poolSocket(const SharedHandle<Request>& request,
+				bool proxyDefined,
+				const SharedHandle<SocketCore>& socket,
+				time_t timeout)
+{
+  if(proxyDefined) {
+    // If proxy is defined, then pool socket with its hostname.
+    poolSocket(request->getHost(), request->getPort(), socket);
+  } else {
+    std::pair<std::string, uint16_t> peerInfo;
+    socket->getPeerInfo(peerInfo);
+    poolSocket(peerInfo.first, peerInfo.second, socket);
+  }
+}
+
+void DownloadEngine::poolSocket
+(const SharedHandle<Request>& request,
+ bool proxyDefined,
+ const SharedHandle<SocketCore>& socket,
+ const std::map<std::string, std::string>& options,				
+ time_t timeout)
+{
+  if(proxyDefined) {
+    // If proxy is defined, then pool socket with its hostname.
+    poolSocket(request->getHost(), request->getPort(), socket, options);
+  } else {
+    std::pair<std::string, uint16_t> peerInfo;
+    socket->getPeerInfo(peerInfo);
+    poolSocket(peerInfo.first, peerInfo.second, socket, options);
+  }
+}
+
 std::multimap<std::string, DownloadEngine::SocketPoolEntry>::iterator
 DownloadEngine::findSocketPoolEntry(const std::string& ipaddr, uint16_t port)
 {
@@ -1016,6 +1071,11 @@ DownloadEngine::popPooledSocket
   return s;
 }
 
+SharedHandle<BtRegistry> DownloadEngine::getBtRegistry() const
+{
+  return _btRegistry;
+}
+
 DownloadEngine::SocketPoolEntry::SocketPoolEntry
 (const SharedHandle<SocketCore>& socket,
  const std::map<std::string, std::string>& options,
@@ -1042,4 +1102,37 @@ DownloadEngine::SocketPoolEntry::getOptions() const
   return _options;
 }
 
+CUID DownloadEngine::newCUID()
+{
+  return _cuidCounter.newID();
+}
+
+const std::string& DownloadEngine::findCachedIPAddress
+(const std::string& hostname) const
+{
+  return _dnsCache->find(hostname);
+}
+
+void DownloadEngine::cacheIPAddress
+(const std::string& hostname, const std::string& ipaddr)
+{
+  _dnsCache->put(hostname, ipaddr);
+}
+
+void DownloadEngine::setAuthConfigFactory
+(const SharedHandle<AuthConfigFactory>& factory)
+{
+  _authConfigFactory = factory;
+}
+
+SharedHandle<AuthConfigFactory> DownloadEngine::getAuthConfigFactory() const
+{
+  return _authConfigFactory;
+}
+
+void DownloadEngine::setRefreshInterval(time_t interval)
+{
+  _refreshInterval = interval;
+}
+
 } // namespace aria2
