@@ -33,6 +33,11 @@
  */
 /* copyright --> */
 #include "MultiUrlRequestInfo.h"
+
+#include <signal.h>
+
+#include <ostream>
+
 #include "RequestGroupMan.h"
 #include "DownloadEngine.h"
 #include "LogFactory.h"
@@ -42,14 +47,17 @@
 #include "DownloadEngineFactory.h"
 #include "RecoverableException.h"
 #include "message.h"
-#include "DNSCache.h"
 #include "Util.h"
 #include "Option.h"
 #include "StatCalc.h"
 #include "CookieStorage.h"
 #include "File.h"
-#include <signal.h>
-#include <ostream>
+#include "Netrc.h"
+#include "AuthConfigFactory.h"
+#ifdef ENABLE_SSL
+# include "SocketCore.h"
+# include "TLSContext.h"
+#endif // ENABLE_SSL
 
 namespace aria2 {
 
@@ -93,11 +101,7 @@ void MultiUrlRequestInfo::printMessageForContinue()
 
 int MultiUrlRequestInfo::execute()
 {
-  {
-    DNSCacheHandle dnsCache(new SimpleDNSCache());
-    DNSCacheSingletonHolder::instance(dnsCache);
-  }
-  int returnValue = 0;
+  int returnValue = 1;
   try {
     DownloadEngineHandle e =
       DownloadEngineFactory().newDownloadEngine(_option, _requestGroups);
@@ -116,6 +120,38 @@ int MultiUrlRequestInfo::execute()
       _logger->error(EX_EXCEPTION_CAUGHT, e);
     }
 
+    SharedHandle<AuthConfigFactory> authConfigFactory
+      (new AuthConfigFactory(_option));
+    File netrccf(_option->get(PREF_NETRC_PATH));
+    if(!_option->getAsBool(PREF_NO_NETRC) && netrccf.isFile()) {
+      mode_t mode = netrccf.mode();
+      if(mode&(S_IRWXG|S_IRWXO)) {
+	_logger->notice(MSG_INCORRECT_NETRC_PERMISSION,
+			_option->get(PREF_NETRC_PATH).c_str());
+      } else {
+	SharedHandle<Netrc> netrc(new Netrc());
+	netrc->parse(_option->get(PREF_NETRC_PATH));
+	authConfigFactory->setNetrc(netrc);
+      }
+    }
+    e->setAuthConfigFactory(authConfigFactory);
+
+#ifdef ENABLE_SSL
+    SharedHandle<TLSContext> tlsContext(new TLSContext());
+    if(_option->defined(PREF_CERTIFICATE) &&
+       _option->defined(PREF_PRIVATE_KEY)) {
+      tlsContext->addClientKeyFile(_option->get(PREF_CERTIFICATE),
+				   _option->get(PREF_PRIVATE_KEY));
+    }
+    if(_option->defined(PREF_CA_CERTIFICATE)) {
+      tlsContext->addTrustedCACertFile(_option->get(PREF_CA_CERTIFICATE));
+    }
+    if(_option->getAsBool(PREF_CHECK_CERTIFICATE)) {
+      tlsContext->enablePeerVerification();
+    }
+    SocketCore::setTLSContext(tlsContext);
+#endif
+
     std::string serverStatIf = _option->get(PREF_SERVER_STAT_IF);
     if(!serverStatIf.empty()) {
       e->_requestGroupMan->loadServerStat(serverStatIf);
@@ -125,7 +161,8 @@ int MultiUrlRequestInfo::execute()
     e->setStatCalc(_statCalc);
     e->fillCommand();
 
-    // The number of simultaneous download is specified by PREF_MAX_CONCURRENT_DOWNLOADS.
+    // The number of simultaneous download is specified by
+    // PREF_MAX_CONCURRENT_DOWNLOADS.
     // The remaining urls are queued into FillRequestGroupCommand.
     // It observes the number of simultaneous downloads and if it is under
     // the limit, it adds RequestGroup object from its queue to DownloadEngine.
@@ -145,13 +182,13 @@ int MultiUrlRequestInfo::execute()
     _summaryOut << std::flush;
 
     RequestGroupMan::DownloadStat s = e->_requestGroupMan->getDownloadStat();
-    if(!s.allCompleted()) {
+    if(s.allCompleted()) {
+      returnValue = 0;
+    } else {
       printMessageForContinue();
-      returnValue = 1;
     }
-  } catch(RecoverableException *ex) {
-    _logger->error(EX_EXCEPTION_CAUGHT, ex);
-    delete ex;
+  } catch(RecoverableException& e) {
+    _logger->error(EX_EXCEPTION_CAUGHT, e);
   }
   Util::setGlobalSignalHandler(SIGINT, SIG_DFL, 0);
   Util::setGlobalSignalHandler(SIGTERM, SIG_DFL, 0);
