@@ -86,7 +86,7 @@ extern const std::string PREF_STARTUP_IDLE_TIME;
 // value: prealloc | none
 extern const std::string PREF_FILE_ALLOCATION;
 extern const std::string V_PREALLOC;
-#// value: 1*digit
+// value: 1*digit
 extern const std::string PREF_NO_FILE_ALLOCATION_LIMIT;
 // value: true | false
 extern const std::string PREF_ALLOW_OVERWRITE;
@@ -159,10 +159,6 @@ extern const std::string PREF_FTP_PASSWD;
 extern const std::string PREF_FTP_TYPE;
 extern const std::string V_BINARY;
 extern const std::string V_ASCII;
-// values: get | tunnel
-extern const std::string PREF_FTP_VIA_HTTP_PROXY;
-extern const std::string V_GET;
-extern const std::string V_TUNNEL;
 // values: true | false
 extern const std::string PREF_FTP_PASV;
 // values: true | false
@@ -188,21 +184,28 @@ extern const std::string PREF_ENABLE_HTTP_PIPELINING;
 extern const std::string PREF_MAX_HTTP_PIPELINING;
 // value: string
 extern const std::string PREF_HEADER;
+// value: string that your file system recognizes as a file name.
+extern const std::string PREF_CERTIFICATE;
+// value: string that your file system recognizes as a file name.
+extern const std::string PREF_PRIVATE_KEY;
+// value: string that your file system recognizes as a file name.
+extern const std::string PREF_CA_CERTIFICATE;
+// value: true | false
+extern const std::string PREF_CHECK_CERTIFICATE;
 
 /**;
- * HTTP proxy related preferences
+ * Proxy related preferences
  */
 extern const std::string PREF_HTTP_PROXY;
-extern const std::string PREF_HTTP_PROXY_USER;
-extern const std::string PREF_HTTP_PROXY_PASSWD;
-extern const std::string PREF_HTTP_PROXY_HOST;
-extern const std::string PREF_HTTP_PROXY_PORT;
+extern const std::string PREF_HTTPS_PROXY;
+extern const std::string PREF_FTP_PROXY;
+extern const std::string PREF_ALL_PROXY;
+// values: comma separeted hostname or domain
+extern const std::string PREF_NO_PROXY;
 // values: get | tunnel
-extern const std::string PREF_HTTP_PROXY_METHOD;
-// values: true | false
-extern const std::string PREF_HTTP_PROXY_ENABLED;
-// values: true | false
-extern const std::string PREF_HTTP_PROXY_AUTH_ENABLED;
+extern const std::string PREF_PROXY_METHOD;
+extern const std::string V_GET;
+extern const std::string V_TUNNEL;
 
 /**
  * BitTorrent related preferences
